
/*
 * Copyright (c) 2009 Silvia Ramis, Marta Ramis
 * Copyright (c) 2016 Jose-Luis Lisani <joseluis.lisani@uib.es>
 * Copyright (c) 2019 Jose-Luis Lisani <joseluis.lisani@uib.es>
 * All rights reserved.
 *
 * This code was originally written by Silvia Ramis and Marta Ramis
 * for their Final Project on Computer Sciences Engineering (2009)
 * under the supervision of J.L. Lisani
 * Some parts of the code were written by J.L. Lisani
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <vector>
#include <algorithm>
using namespace std;

//Classifier data
//Data for each feature-mask 
struct classifier_item {
    int x1, y1, x2, y2; //position in 24x24 image
    int type; //1=2V, 2=2H, 3=3V, 4=3H, 5=4
    float vth; //classification threshold
    int polth; //threshold sign
    float alpha;//learned alpha value
};


//Read classifier data:
//* number of features masks
//* for each mask:
//  - position in 24x24 image
//  - type
//  - classification threshold
//  - threshold sign
//  - learned alpha value
vector<classifier_item> read_classifier_data(const char *name);


//Compute output value of the classifier when applied on a subimage
//standard subimage size: same size as training samples (default: 24 x 24)
//return 0 if subimage is flat
//else return 1 and the output value of the classifier 
unsigned char check_features_imageStandard(unsigned char *I, int w, int h, 
                                           float *If, float *Ii,
                                           vector<classifier_item> cdata, 
                                           float &vdet);



